<div class="modal fade" id="modal-container-{{obj.test_id}}" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true">
    <div class="modal-dialog modal-xl" role="document">
        <div class="modal-content">
            <div class="modal-header">
                <h5 class="modal-title" id="myModalLabel">
                    {{obj.test}}
                </h5> 
                <button type="button" class="close" data-dismiss="modal">
                    <span aria-hidden="true">×</span>
                </button>
            </div>
            <div class="modal-body">
                {% if obj.error_info %}
                    <h6 class="text-primary">Error Info</h6>
                    <p class="text-danger" style="white-space: pre-wrap">Validation Failed For: {{obj.error_info}}</p>
                {% endif %}
                
                <h6 class="text-primary">Request Body</h6>
                <div class="container-boxes">
                    <pre><code class="text-info">{{obj.request_body}}</code></pre>
                </div>
                <br>

                <h6 class="text-primary">Header</h6>
                <div class="container-boxes">
                    <pre><code class="text-info">{{obj.request_header}}</code></pre>
                </div>
                <br>
                
                    <div class="row">
                        <div class="col-md-6">
                            <h6 class="text-primary">Actual Response</h6>
                                <small>
                                    <div class="container-boxes">
                                        <pre><code class="text-info">{{obj.response|safe}}</code></pre></small>
                                    </div>
                                
                        </div>
                        <div class="col-md-6">
                            <h6 class="text-primary">Expected Response</h6>
                            <small>
                                <div class="container-boxes">
                                <pre class="lang-json"><code class="text-info">{{obj.expected_response|safe}}</code></pre>
                            </div></small>
                        </div>

                    </div>
                
                <br>

            </div>
            <div class="modal-footer">
                <button type="button" class="btn btn-secondary" data-dismiss="modal">
                    Close
                </button>
            </div>
        </div>
        
    </div>
    
</div>
    

