
<!DOCTYPE html>
<html lang="en">
	

<head>
	<meta charset="utf-8">
	<meta http-equiv="X-UA-Compatible" content="IE=edge">
	<meta name="viewport" content="width=device-width, initial-scale=1">

	<title>Test Report</title>

	<link href="https://stackpath.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css" rel="stylesheet"
		integrity="sha384-ggOyR0iXCbMQv3Xipma34MD+dH/1fQ784/j6cY/iJTQUOhcWr7x9JvoRxT2MZw1T" crossorigin="anonymous">
	<style>
		body {
			width: 70%;
			padding-top: 50px;
			margin: auto;
		}

		.container-boxes {
			max-height: 250px; 
			overflow: auto;  
			border-radius: 5px; 
			border: 2px solid #EBF6F6 ; 
			padding: 15px;
		}
	</style>
	<script src="https://cdn.jsdelivr.net/gh/google/code-prettify@master/loader/run_prettify.js"></script>

</head>

<body>

	<div class="container-fluid">

		<div class="row">
			<div class="col-md-3">
			</div>
			<div class="col-md-8">
				<h1 class="text-primary">Automated Test Report<h1>
			</div>
			<div class="col-md-1">
			</div>
		</div>
		<br>

		<div class="row">
			<div class="col-md-12">
				<h4>Summary</h4>
			</div>
			<br>
			<br>
			<div class="col-md-4">
				<div class="card text-white" style="background-color: #408751;">
					<h5 class="card-header">
						SUCCESS
					</h5>
					<div class="card-body">
						<p class="card-text">
							{{success_count|default:"0"}}
						</p>
					</div>
					<div class="card-footer">
						{{success_percentage|default:"0"}}%
					</div>
				</div>
			</div>
			<div class="col-md-4">
				<div class="card text-white" style="background-color: #CF573D;">
					<h5 class="card-header">
						FAILED
					</h5>
					<div class="card-body">
						<p class="card-text">
							{{fail_count|default:"0"}}
						</p>
					</div>
					<div class="card-footer">
						{{fail_percentage|default:"0"}}%
					</div>
				</div>
			</div>
			<div class="col-md-4">
				<div class="card text-white" style="background-color: #A08A78">
					<h5 class="card-header">
						EXCEPTIONS
					</h5>
					<div class="card-body">
						<p class="card-text">
							{{exception_count|default:"0"}}
						</p>
					</div>
					<div class="card-footer">
						{{exception_percentage|default:"0"}}%
					</div>
				</div>
			</div>
		</div>
		<br>
		{% if priority_fail_count %}
			<div class="row">
				<div class="col-md-12">
					<h4>Priority Wise Fail Count</h4>
				</div>
				<br>
				<br>
				<div class="col-md-12">
					<table class="table table-striped table-dark">
						<thead>
							<tr>
								<th>
									Priority
								</th>
								<th>
									Failed
								</th>
							</tr>
						</thead>
						<tbody>
							{% for row in priority_fail_count %}
								<tr>
									<td>{{row.1}}</td>
									<td>{{row.0}}</td>
								</tr>
							{% endfor %}
						</tbody>
					</table>
				</div>
			</div>
		{% endif %}

		<br>


		{% if failed_tests %}
			<div class="col-md-12">
				<h4>Failed Tests</h4>
			</div>
			<br>

			<div id="card-3">

				{% for obj in failed_tests %}
					{% include "test_details.html" with obj=obj %}					
				{% endfor %}
			</div>
		</br>
		{% endif %}


		{% if passed_tests %}
			<div class="col-md-12">
				<h4>Passed Tests</h4>
			</div>
			<br>
	
			<div id="card-3">

				{% for obj in passed_tests %}
					{% include "test_details.html" with obj=obj %}					
				{% endfor %}
			</div>
		</div>
	</br>
		{% endif %}

		{% include "exceptions.html" with exception_details=exception_details %}
			
	</div>
                
	<br><br><br>

	<script src="https://code.jquery.com/jquery-3.2.1.slim.min.js"
		integrity="sha384-KJ3o2DKtIkvYIK3UENzmM7KCkRr/rE9/Qpg6aAZGJwFDMVNA/GpGFF93hXpG5KkN" crossorigin="anonymous">
	</script>
	<script src="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0/js/bootstrap.min.js"
		integrity="sha384-JZR6Spejh4U02d8jOt6vLEHfe/JQGiRRSQQxSfFWpi1MquVdAyjUar5+76PVCmYl" crossorigin="anonymous">
	</script>

</body>

</html>